angular.module('sdw.localstorage', ['sdw.system', 'ngCordova'])
    .service('LC', function (system, $cordovaFile, $q) {
        var that = this;

        this.get = function (key) {
            var deferred = $q.defer();
            that.readStorage().then(function (storage) {
                if (key == 'savedforms') {
                    system.debug('LOCALSTORAGE GET ' + key, JSON.stringify(storage));
                    system.debug('LOCALSTORAGE GET ' + key, JSON.stringify(storage[key]));
                }

                if (storage[key] != 'undefined' && storage[key] != "" && storage[key] != null) {
                    deferred.resolve(storage[key]);
                } else {
                    that.set(key, []).then(function () {
                        deferred.resolve([]);
                    });
                }
            });
            return deferred.promise;
        };

        this.set = function (key, value) {
            // var deferred = Promise.defer();
            var deferred = $q.defer();
            that.readStorage().then(function (storage) {
                storage[key] = value;
                that.writeStorage(storage).then(function () {
                    deferred.resolve(storage[key]);
                });
            });
            return deferred.promise;
        };

        this.delete = function (key, idKey, idValue) {
            // var deferred = Promise.defer();
            var deferred = $q.defer();
            that.get(key).then(function (values) {

                angular.forEach(values, function (value, key2) {
                    if (values[key2][idKey] && values[key2][idKey] == idValue)
                        values.splice(key2, 1);
                });

                that.set(key, values).then(function (storage) {
                    deferred.resolve(storage);
                });
            });
            return deferred.promise;
        };

        this.getElement = function (key, idKey, idValue) {
            // var deferred = Promise.defer();
            var deferred = $q.defer();
            that.get(key).then(function (values) {
                for (var value in values) {
                    if (values[value][idKey] && values[value][idKey] == idValue)
                        deferred.resolve(values[value]);
                }

                deferred.reject('no value found');
            });
            return deferred.promise;
        };

        this.cleanAll = function () {
            that.writeStorage({
                savedforms: []
            });
        };


        this.init = function () {


            var deferred = $q.defer();
            try {

                document.addEventListener("deviceready", function () {
                    window.requestFileSystem(window.LocalFileSystem.PERSISTENT, 0, function (fs) {

                        fs.root.getFile("umax-storage.txt", {
                            create:    false,
                            exclusive: false
                        }, function (fileEntry) {

                            if (!fileEntry.isFile) {

                                that.writeStorage({
                                    "test":       "test",
                                    "natures":    [],
                                    "savedforms": []
                                });
                            } else {

                                that.readStorage();
                            }
                            deferred.resolve();
                        }, function (error) {
                            var data = {
                                "test":       "test",
                                "natures":    [],
                                "savedforms": []
                            };
                            that.writeStorage(data);

                            deferred.resolve(data);
                        });

                    }, function (error) {
                        console.log("ERROR READING FS : " + JSON.stringify(error));
                        deferred.reject('impossible de lire le stockage local');
                    });
                });
            } catch (e) {
                console.log("EXCEPTION : " + JSON.stringify(e));
                deferred.reject('lib file not active');
            }
            return deferred.promise;
        };

        this.readStorage = function () {

            // var deferred = Promise.defer();
            var deferred = $q.defer();
            try {
                window.requestFileSystem(1, 0, function (fs) {
                    fs.root.getFile("umax-storage.txt", {
                        create:    true,
                        exclusive: false
                    }, function (fileEntry) {

                        console.log("fileEntry is file?" + fileEntry.isFile.toString());
                        readFile(fileEntry, function (txt) {
                            var data = txt;

                            console.log('DATA READ : ' + data);

                            if (data.length == 0) {
                                data = '{"test":"test", "natures":[], "savedforms": []}';
                            }

                            try
                            {
                                deferred.resolve(JSON.parse(data));
                            }
                            catch (error){

                                data=cleanJson(data);

                                try
                                {
                                    deferred.resolve(JSON.parse(data));
                                }
                                catch (error)
                                {
                                    data = '{"test":"test", "natures":[], "savedforms": []}';
                                    deferred.resolve(JSON.parse(data));
                                }
                            }
                        });

                    }, function (error) {
                        console.log("ERROR READING FILE : " + JSON.stringify(error));
                        // alert('impossible de lire le stockage local');
                        // deferred.reject('impossible de lire le stockage local');
                        var data = {
                            "natures":    [],
                            "savedforms": []
                        };
                        that.writeStorage(data);

                        deferred.resolve(data);
                    });

                }, function (error) {
                    console.log("ERROR READING FS : " + JSON.stringify(error));
                    deferred.reject('impossible de lire le stockage local');
                });
            } catch (e) {
                console.log("EXCEPTION : " + JSON.stringify(e));
                deferred.reject('lib file not active');
            }

            return deferred.promise;
        };

        this.writeStorage = function (storage) {

            var deferred = $q.defer();
            try {
                window.requestFileSystem(1, 0, function (fs) {
                    fs.root.getFile("umax-storage.txt", {
                        create:    true,
                        exclusive: false
                    }, function (fileEntry) {

                        console.log("fileEntry is file?" + fileEntry.isFile.toString());
                        writeFile(fileEntry, JSON.stringify(storage), function () {
                            deferred.resolve(storage);
                        });

                    }, function (error) {
                        console.log("ERROR WRITING FILE : " + JSON.stringify(error));
                        alert('impossible d\'écrire le stockage local');
                        deferred.reject('impossible d\'écrire le stockage local');
                    });

                }, function (error) {
                    console.log("ERROR WRITING FS : " + JSON.stringify(error));
                    deferred.reject('impossible d\'écrire le stockage local');
                });
            } catch (e) {
                deferred.reject('lib file not active');
            }

            return deferred.promise;
        };

        function writeFile(fileEntry, dataObj, callback) {
            // Create a FileWriter object for our FileEntry (log.txt).
            try {
                fileEntry.createWriter(function (fileWriter) {

                    fileWriter.onwriteend = function () {
                        console.log("Successful file write...");
                        callback();
                    };

                    fileWriter.onerror = function (e) {
                        console.log("Failed file write: " + e.toString());
                    };

                    // If data object is not passed in,
                    // create a new Blob instead.
                    if (!dataObj) {
                        dataObj = new Blob(['some file data'], {type: 'text/plain'});
                    }

                    fileWriter.write(dataObj);
                });
            } catch (e) {
                console.log("EXCEPTION : " + JSON.stringify(e));
            }
        }

        function readFile(fileEntry, callback) {
            try {
                fileEntry.file(function (file) {
                    var reader = new FileReader();

                    reader.onloadend = function () {
                        console.log("Successful file read: " + this.result);
                        callback(this.result);
                    };

                    reader.readAsText(file);

                }, function (error) {
                    console.log("ERROR READ FILE : " + JSON.stringify(error));
                });
            } catch (e) {
                console.log("EXCEPTION : " + JSON.stringify(e));
            }
        }

        function cleanJson(dirtyJson)
        {
            var cleanJson="";
            var inQuote=false;
            var escaped=false;
            var level=0;

            for(var i=0;i<dirtyJson.length;i++)
            {
                var car=dirtyJson[i];

                cleanJson+=car;

                if(car==="{" && !inQuote)
                {
                    level++;
                }
                else if(car==="}" && !inQuote)
                {
                    level--;
                    if(level===0)break;
                }
                else if(car==="\"")
                {
                    if(!inQuote)
                        inQuote=true;
                    else if(inQuote && !escaped)
                        inQuote=false;
                    else if(inQuote && escaped)
                        escaped=false;
                }
                else if(car==="\\")
                {
                    if(inQuote && !escaped)
                    {
                        escaped=true;
                    }
                    else if(inQuote && escaped)
                    {
                        escaped=false;
                    }
                }
                else if(inQuote && escaped)
                {
                    escaped=false;
                }
            }

            if(escaped)cleanJson+="\"";
            if(inQuote)cleanJson+="\"";
            while(level>0)
            {
                cleanJson+="}";
                level--;
            }

            return cleanJson;
        }
    });