angular.module('sdw.system', [])
    .service('system', function ($ionicHistory, $ionicModal, $ionicLoading, $ionicPopup, $rootScope) {
        var that = this;
        this.forward = function ($state, disableBack, route, params) {
            $ionicHistory.nextViewOptions({
                disableBack: disableBack
            });
            $state.go(route, params);
        };
        this.createModal = function ($scope, template, scopeHolder, autoOpen) {
            return $ionicModal.fromTemplateUrl(template, {
                scope: $scope
            }).then(function (modal) {
                $scope[scopeHolder] = modal;
                if (autoOpen)
                    $scope[scopeHolder].show();
            });
        };
        this.showLoading = function () {
            $ionicLoading.show({
                content:           '<i class="icon ion-loading-c"></i>',
                animation:         'fade-in',
                noBackdrop:        true,
                maxWidth:          50,
                showDelay:         0,
                hideOnStateChange: true,
                duration:          2000
            });
        };
        this.hideLoading = function () {
            $ionicLoading.hide();
        };
        this.alert = function (title, content, callback) {
            $ionicPopup.alert({
                title:    title,
                template: content
            }).then(function (res) {
                if (callback)
                    callback();
            });
        };

        this.confirm = function (text, callback) {
            $ionicPopup.confirm({
                title:      '',
                template:   text,
                cancelText: 'ABANDON',
                okText:     'OK'
            }).then(function (res) {
                // console.log(res);
                // console.log(callback);
                if (res && callback)
                    callback();
            });
        };

        this.ask = function (text, inputLabel, callback, failedCallback) {
            $rootScope.ask = {input: ''};
            $ionicPopup.show({
                template: '<input type="text" ng-model="ask.input">',
                title:    text,
                subTitle: inputLabel,
                scope:    $rootScope,
                buttons:  [
                    {
                        text: 'Non',
                        type: 'button-positive'
                    },
                    {
                        text:  '<b>OK</b>',
                        type:  'button-energized',
                        onTap: function (e) {
                            if (!$rootScope.ask.input) {
                                e.preventDefault();
                                alert('Vous devez saisir une valeur');
                            } else {
                                return $rootScope.ask.input;
                            }
                        }
                    }
                ]
            }).then(function (res) {
                if (res && callback)
                    callback(res);
                else
                    failedCallback();
                return res;
            });
        };

        this.call = function ($state, route) {
            $state.go(route);
        };

        this.debug = function (message, value) {
            if (settings.env == 'dev') {
                console.info("[DEBUG] " + message);
                if (Array.isArray(value)) {
                    for (var i = 0; i < value.length; i++) {
                        this.printTruncatedLog(JSON.stringify(value[i]));
                    }
                } else if (angular.isObject(value)) {
                    this.printTruncatedLog(JSON.stringify(value));
                } else {
                    this.printTruncatedLog(value);
                }
            }
        };

        this.printTruncatedLog = function (message) {
            if (typeof message != 'undefined' && message != null && message.length > 4000) {
                console.info("[DEBUG] " + message.substr(0, 4000));
                this.printTruncatedLog(message.substr(4000));
            } else {
                console.info("[DEBUG] " + message);
            }
        };
        this.webservice = function ($http, name, data, then) {
            if (data === null) {
                data = {};
            }

            var dataString = "";

            for (index in data) {
                var dataValue;
                if (typeof(data[index]) == "boolean") {
                    dataValue = +data[index];
                } else if (data[index] === null || typeof(data[index]) == "undefined") {
                    dataValue = '';
                    data[index] = '';
                } else if (typeof(data[index]) == "object") {
                    dataValue = "Object";
                } else {
                    dataValue = data[index];
                }
                dataString += "|" + index + ":" + dataValue;
            }

            if (settings.env == 'dev') {
                that.debug('WS URL : ' + settings.webServiceBaseUrl + name, dataString);
                that.debug('WS DATA : ', data);
            }

            var sign = md5("4toXvyq_9rQeRQl5Sak6rKXmrDsPx3p5skTuNvwVZt1fqnRd4a" + dataString + "q--qkbNkYBK6qvqK3NjWqicFwHunk7Nw");

            $http.post(settings.webServiceBaseUrl + name + '&sign=' + sign, data).then(then)
        };
        this.checkConnection = function () {
            try {
                var networkState = navigator.connection.type;

                var states = {};
                states[Connection.UNKNOWN] = 'Unknown connection';
                states[Connection.ETHERNET] = 'Ethernet connection';
                states[Connection.WIFI] = 'WiFi connection';
                states[Connection.CELL_2G] = 'Cell 2G connection';
                states[Connection.CELL_3G] = 'Cell 3G connection';
                states[Connection.CELL_4G] = 'Cell 4G connection';
                states[Connection.CELL] = 'Cell generic connection';
                states[Connection.NONE] = 'No network connection';

                that.debug('Connection type: ', states[networkState]);
            } catch (e) {
                that.debug('Connection Exception: ', e);
            }
        };

        this.isConnected = function () {
            try {
                var networkState = navigator.connection.type;

                return networkState != Connection.NONE && networkState != Connection.UNKNOWN;
            } catch (e) {
                // that.debug('Connection Exception: ', e);
                return true;
            }
        };

        $rootScope.system = this;
    }).directive('autofocus', ['$timeout', function ($timeout) {
    return {
        restrict: 'A',
        link:     function ($scope, $element) {
            $timeout(function () {
                $element[0].focus();
            });
        }
    }
}]);