angular.module('sdw.services', ['sdw.tools', 'sdw.system'])
    .service('ws', function ($http, $state, $rootScope, tools, system, LC) {

            var debug = true;

            function wsError(message) {
                system.alert(message);
                return true;
            }

            this.sessionStart = function () {
                system.webservice($http, "User_Session", null, function (response) {
                    var result, error;
                    if (response.data.type == "error") {
                        error = wsError(response.data.content);
                    } else {
                        result = response.data.content;
                    }
                    if (!error) {
                        LC.sessionToken = response.data.sessionToken;
                    }
                });
            };

            /*
             Performs user login
             */
            this.logInAction = function ($scope, email, password, rememberMe, callback) {
                if (tools.isDefined(email) && tools.isDefined(password)) {
                    system.showLoading();
                    system.webservice($http, "User_Login", {
                        'email':    email,
                        'password': tools.base64_encode(password)
                    }, function (response) {
                        var result, error;
                        if (response.data.type == "error") {
                            error = wsError(response.data.content);
                        } else {
                            result = response.data.content;
                        }

                        system.debug('WS RESPONSE : ', response);

                        $rootScope.user = result.user;

                        if ($rootScope.user.id !== 0 && !error) {

                            if (rememberMe) {
                                result.user.password = password;
                                $scope.setRememberedUser(result.user);
                            } else {
                                $scope.setRememberedUser(null);
                            }
                            $scope.closeLogin();

                            $rootScope.user = result.user;

                            if (callback)
                                callback($scope);

                            system.webservice($http, "Forms_ListNatures", {}, function (response) {
                                var result, error;
                                if (response.data.type == "error") {
                                    error = wsError(response.data.content);
                                } else {
                                    result = response.data.content;
                                }

                                system.debug('GET NATURES WS', response.data.content);
                                // window.localStorage['natures'] = JSON.stringify(result.natures);
                                $rootScope.natures = result.natures;
                                LC.set('natures', result.natures);

                                system.hideLoading();
                            });

                        } else {
                            $rootScope.user = null;
                            system.alert(
                                'Problème de connexion',
                                'Nom d\'utilisateur et/ou mot de passe incorrect'
                            );
                        }
                        system.hideLoading();
                    });
                }
            };


            /*
             Creates new user
             */
            this.createNewUser = function ($scope, newUser, route) {
                system.showLoading();
                system.webservice($http, "User_CreateUser", {
                    'nom':      newUser.nom,
                    'prenom':   newUser.prenom,
                    'email':    newUser.email,
                    'password': newUser.password
                }, function (response) {
                    var result, error;
                    if (response.data.type == "error") {
                        error = wsError(response.data.content);
                    } else {
                        result = response.data.content;
                    }
                    if (!error) {
                        system.alert(
                            'Félicitations',
                            result
                        );

                        system.hideLoading();

                        system.forward($state, true, 'app.home');
                    }
                });
            };

            /*
             Saves user profile values
             */
            this.saveUser = function (user) {
                system.showLoading();

                system.webservice($http, "User_SaveUser", {
                    'id':     user.id,
                    'email':  user.email,
                    'nom':    user.nom,
                    'prenom': user.prenom
                }, function (response) {
                    var result, error;
                    if (response.data.type == "error") {
                        error = wsError(response.data.content);
                    } else {
                        result = response.data.content;
                    }

                    if (!error) {
                        $rootScope.user.email = result.user.email;
                        $rootScope.user.nom = result.user.nom;
                        $rootScope.user.prenom = result.user.prenom;

                        if ($rootScope.getRememberedUser() !== null)
                            $rootScope.setRememberedUser($rootScope.user);
                    }
                    system.hideLoading();
                });
            };

            /*
             Performs user login
             */
            this.getListNatures = function (callback) {
                system.showLoading();

                $rootScope.natures = null;

                LC.get('natures').then(function (natures) {

                    system.debug('LOCAL STORAGE NATURES', natures);

                    if (!system.isConnected() && natures != "" && natures != null) {
                        if (natures == "[object Object]") {
                            LC.set('natures', null);
                        }

                        $rootScope.natures = natures;
                    }

                    if ($rootScope.natures == null) {
                        system.webservice($http, "Forms_ListNatures", {}, function (response) {

                            var result, error;
                            if (response.data.type == "error") {
                                error = wsError(response.data.content);
                            } else {
                                window.localStorage['fetched'] = true;
                                result = response.data.content;
                            }

                            $rootScope.natures = result.natures;

                            // window.localStorage['natures'] = JSON.stringify(result.natures);
                            LC.set('natures', result.natures);

                            system.hideLoading();
                        });
                    }
                }, function (erreur) {

                    system.debug('INDEX NATURES', erreur);
                });
            };

            /*
             Performs user login
             */
            /*
             Performs submitNature
             */


            this.getNature = function ($scope, natureId, callback) {
                system.showLoading();

                var natures = LC.get('natures').then(function (natures) {

                    system.debug('SYS CONN', system.isConnected());

                    if (!system.isConnected()) {

                        if (natures != "" && natures != null) {
                            if (natures == "[object Object]") {
                                LC.set('natures', null);
                            }

                            $rootScope.natures = natures;
                        }

                        angular.forEach(natures, function (value, key) {
                            if (value.id_nature == natureId) {
                                $scope.nature = value;

                                system.debug('CURRENT NATURE', value);

                                $scope.nbEtapes = value.pages.length;

                                system.hideLoading();
                            }
                        });
                        system.hideLoading();
                    } else {
                        if ($scope.nature == null) {
                            system.webservice($http, "Forms_GetNature", {"natureId": natureId}, function (response) {
                                var result, error;
                                if (response.data.type == "error") {
                                    error = wsError(response.data.content);
                                } else {
                                    result = response.data.content;
                                }

                                $scope.nature = result.nature;
                                $scope.nbEtapes = $scope.nature.pages.length;

                                $scope.checkAttributes();

                                system.hideLoading();
                            });
                        }
                    }


                }, function () {
                    if ($scope.nature == null) {
                        system.webservice($http, "Forms_GetNature", {"natureId": natureId}, function (response) {
                            var result, error;
                            if (response.data.type == "error") {
                                error = wsError(response.data.content);
                            } else {
                                result = response.data.content;
                            }

                            $scope.nature = result.nature;
                            $scope.nbEtapes = $scope.nature.pages.length;

                            $scope.checkAttributes();

                            system.hideLoading();
                        });
                    }
                });


            };
            this.submitNature = function ($scope, nature, callback) {
                system.showLoading();

                console.log('SUBMITTING FORM VALUES PDF');
                console.log(JSON.stringify(nature));
                system.webservice($http, "Forms_Submit", {
                    'data':    JSON.stringify(nature),
                    'user_id': $rootScope.user.id
                }, function (response) {
                    var result, error;
                    if (response.data.type == "error") {
                        error = wsError(response.data.content);
                    } else {
                        result = response.data.content;

                        if (result.nature != '') {
                            if (callback)
                                callback($scope);

                            system.alert(
                                'Formulaire envoyé',
                                'Votre formulaire a bien été envoyé'
                            );
                            system.forward($state, true, 'app.home');
                        } else {
                            system.alert(
                                'Problème de connexion',
                                'Identifiant utilisateur incorrect'
                            );
                        }
                    }
                    system.hideLoading();
                });

            };
        }
    );