angular.module('sdw.login', ['sdw.services', 'sdw.system', 'sdw.tools'])
    .service('login', function ($rootScope, $state, ws, system, tools) {
        this.initLogin = function () {

            if (!tools.isDefined($rootScope.user) || Object.keys($rootScope.user).length == 0)
                $rootScope.user = null;

            system.createModal($rootScope, 'templates/modales/login.html', 'modal', false);

            var cb = null;

            // Open the login modal
            $rootScope.login = function (callback) {
                //console.log(showFlashMessages());
                cb = callback;
                $rootScope.modal.show();
            };

            // disconnect the user
            $rootScope.logout = function () {
                $rootScope.user = null;
                $rootScope.setRememberedUser(null);
                system.forward($state, true, 'app.home');
            };

            // Check if user is logged in or not
            $rootScope.isLoggedIn = function () {
                return $rootScope.user != null;
            };

            // Form data for the login modal
            $rootScope.loginData = {};

            // Triggered in the login modal to close it
            $rootScope.closeLogin = function () {
                $rootScope.modal.hide();
            };

            // Perform the login action when the user submits the login form
            $rootScope.doLogin = function () {

                if (!$rootScope.loginData.email || !$rootScope.loginData.password) {
                    system.alert('Erreur de données', 'Vous devez renseigner les champs obligatoires (marqués d\'une <sup>*</sup>)');
                    return null;
                }
                ws.logInAction(
                    $rootScope,
                    $rootScope.loginData.email,
                    $rootScope.loginData.password,
                    $rootScope.loginData.rememberMe,
                    cb
                );
            };

            // Retrieve the remembered user stored in local storage
            $rootScope.getRememberedUser = function () {
                if (window.localStorage['rememberedUser'] != "" && window.localStorage['rememberedUser'] != null) {
                    if (window.localStorage['rememberedUser'] == "[object Object]") {
                        window.localStorage['rememberedUser'] = null;
                    }

                    return JSON.parse(window.localStorage['rememberedUser']);
                } else {
                    return null;
                }
            };

            // Stores the remembered user in local storage
            $rootScope.setRememberedUser = function (user) {
                window.localStorage['rememberedUser'] = JSON.stringify(user);
            };

            // Identify remembered user if set
            $rootScope.reconnect = function () {

                var user = $rootScope.getRememberedUser();

                system.debug('REMEMBERED USER', user);

                if (user != null) {

                    $rootScope.loginData.email = user.email;
                    $rootScope.loginData.password = user.password;
                    $rootScope.loginData.rememberMe = true;

                    system.debug('REMEMBERED USER SCOPE', $rootScope.loginData);

                    if (!system.isConnected()) {
                        $rootScope.user = user;
                    }
                    else {
                        ws.logInAction(
                            $rootScope,
                            user.email,
                            user.password,
                            true,
                            function () {
                                $rootScope.user = user;
                            }
                        );
                    }
                }
            };

            $rootScope.reconnect();
        };

    });